@@ -688,9 +688,30 @@ proc event_client_start {} {
 # event_echo
 doc add "Events ($doc_type, Script)/$doc_section/event_echo" "event_echo <message> <pathname> <new>
 
-This event handler performs scrollback size limiting, URL highlighting, and timestamping."
+This event handler performs windowlist nick highlighting, scrollback size limiting, URL highlighting, and timestamping."
 
 proc event_echo { message pathname new } {
+    # Highligting on nick
+    if { $::dynamic::windowlist_nick_highlight } {
+        if { $pathname != ".main" } {
+	    set nick [evalserver [serverindex $pathname] mynick]
+	    regsub -all {\W} $nick {\\&} escnick
+	    # Lowercase them so we match different cases
+	    set lcmess [string tolower $message]
+	    set escnick [string tolower $escnick]
+	    # Crazy regexp to try and match nicks only when they're not part of
+	    # other words.
+	    set expr {[^][`{}a-z0-9_^-]}
+	    if { [regexp "($expr|^)${escnick}($expr|\$)" $lcmess] } {
+		# Add in the special coloring for the nick highlighting.  This
+		# doesn't really need to be done here, but here it is anyways.
+                windowlist_addcolor special 2000 $::dynamic::theme_windowlist_specialforeground $::dynamic::theme_windowlist_specialbackground
+
+		windowlist_colorize $pathname special
+            }
+        }
+    }
+
     # Scrollback size limiting
     if { $::dynamic::scrollback_length } {
 	if { [$pathname.text index end] >
@@ -700,12 +721,14 @@ proc event_echo { message pathname new } {
 	    $pathname.text delete $bl.0 [expr $bl+1].0
 	}
     }
+
     # URL highlighting
     if { $::dynamic::url_highlighting } {
 	set messagetags [URLparse $message $pathname]
     } else {
 	set messagetags [gettags $message]
     }
+
     # Timestamp system
     if { [info tclversion]>=8.3 || $::dynamic::timestamp } {
 	set timestamp [addtag [gettags "[clock format [clock seconds] -format $::dynamic::timestamp_format] " $pathname] timestamp]
@@ -1369,6 +1392,29 @@ proc URLparse { message pathname } {
     return $templ
 }
 
+doc add "Procedures ($doc_type)/$doc_section/windowlist_addcolor" "windowlist_addcolor <name> <priority> <fg color> \[<bg color>]
+
+Adds a color for use in the windowlist.  Once you have added a color, you can use it to color entries in the windowlist.  The priority is a number that indicates the importance of the color.  The 0 color is special and should generally not be used.  The 'active' color has a priority of 1000.  If your color is more important than the active color, give it a higher priority, for example 2000.  If you want your color to be overwritten when the window becomes active, use a priority such as 500.  The name for the color is just used to identify the color combination.  You must specify a valid Tk color for the foreground and background colors.  The background color is optional."
+
+proc windowlist_addcolor { name priority args } {
+    if { [llength $args] == 1 } {
+        QListBox::addcolor .windowlist $name $priority [lindex $args 0]
+    } elseif { [llength $args] == 2 } {
+        QListBox::addcolor .windowlist $name $priority [lindex $args 0] [lindex $args 1]
+    } else {
+	error "Invalid number of arguments passed to windowlist_addcolor."
+    }
+}
+
+doc add "Procedures ($doc_type)/$doc_section/windowlist_colorize" "windowlist_colorize <pathname> <name>
+
+Colors the entry in the window list that corresponds to the given pathname.  The name must be one of the colors added by windowlist_addcolor.  Note that the window list entry will only be colored if the priority of the color is higher than the priority of the existing color."
+
+proc windowlist_colorize { pathname name } {
+    QListBox::colorize .windowlist [windowindex $pathname] $name
+}
+
+
 # SERVER SPECIFIC PROCEDURES
 
 # ::template::ban
@@ -1596,6 +1642,7 @@ proc mousewheel {mod button x y} {
     }
 }
     
+
 # Make AppIcons work... maybe not.
 #wm command . quirc
 #wm group . .
