@@ -201,7 +201,7 @@ proc convert_bg { from to value script } {
 	foreach item $path {
 	    set value [eval $::convert_code($item)]
 	}
-	set script [parseformat $script [list [list e $value]]]
+	set script [parseformat $script [list [list e [escape $value]]]]
 	eval $script
     } else {
 	# We have no direct path, try the currency convertor.
