@@ -3,6 +3,8 @@ proc GUI_Init_Phase_1 {} {
     frame .mainwindow
     # Create the Window List
     QListBox::create .windowlist totop -background $::dynamic::theme_windowlist_background -selectbackground $::dynamic::theme_windowlist_selectbackground -selectforeground $::dynamic::theme_windowlist_selectforeground -foreground $::dynamic::theme_windowlist_foreground -width $::dynamic::theme_windowlist_width -font $::dynamic::theme_windowlist_font
+    QListBox::addcolor .windowlist normal 0 $::dynamic::theme_windowlist_foreground $::dynamic::theme_windowlist_background
+    QListBox::addcolor .windowlist active 1000 $::dynamic::theme_windowlist_activeforeground $::dynamic::theme_windowlist_activebackground
 }
 
 proc GUI_Init_Phase_2 {} {
@@ -303,6 +305,10 @@ namespace eval QPasswordBox {
 
 namespace eval QListBox {
     set tagindex 0
+    array set colors {}
+    array set priorities {}
+    array set selected {}
+    
     proc killtags { windowname first } {
 	foreach tagtype [$windowname tag names $first] {
 	    if { $tagtype != "selected" } {
@@ -317,6 +323,8 @@ namespace eval QListBox {
 	set selectforeground black
         set font "helvetica"
         set width 100
+        set ::QListBox::priorities($windowname) {}
+        set ::QListBox::selected($windowname) -1
 	for { set n 0 } { $n < [llength $args] } { incr n } {
 	    switch -- [lindex $args $n] {
 		-background {
@@ -371,6 +379,7 @@ namespace eval QListBox {
 	$windowname.t insert $index.0 "$item\n"
 	$windowname.t delete "end - 2 c" end
 	$windowname.t configure -state disabled
+	set ::QListBox::priorities($windowname) [linsert $::QListBox::priorities($windowname) [expr $index - 1] 0]
         #$windowname.t tag remove selected 0.0 end
 	#$windowname.t tag add selected $index.0 $index.end
     }
@@ -381,24 +390,55 @@ namespace eval QListBox {
 	$windowname.t delete $index.0 [expr $index+1].0
 	$windowname.t delete "end - 2 c" end
         $windowname.t configure -state disabled
+	set ::QListBox::priorities($windowname) [lreplace $::QListBox::priorities($windowname) [expr $index - 1] [expr $index - 1]]
     }
-    proc colorize { windowname index args } {
+    proc addcolor { windowname name priority args } {
+        if { [llength $args] != 1 && [llength $args] != 2 } {
+            error "Usage: QListBox::addcolor <name> <priority> <foreground> \[<background>\]"
+	}
+        set ::QListBox::colors($name) [list $priority $args]
+    }
+    proc colorize { windowname index name } {
+        # If we're working with the selected window, return.  We don't want to
+	# return when we're killing the color though.
+        if { $::QListBox::selected($windowname) == $index &&
+             $name != "normal" } { return }
+
+        if { [array names ::QListBox::colors -exact $name] == "" } {
+	    error "The $name color was not previously added with addcolor."
+        }
+
 	incr index
 	if { $index>=[lindex [split [$windowname.t index end] "."] 0] || $index<1} {
 	    error "Illegal window index given to colorize"
-	    return;
 	}
+
+        # Extract the colors and priority
+	set priority [lindex $::QListBox::colors($name) 0]
+	set cols [lindex $::QListBox::colors($name) 1]
+
+        # Grab the priority from the priority list for this window.
+        set prilist $::QListBox::priorities($windowname)
+	set oldpriority [lindex $prilist [expr $index - 1]]
+
+        # Only if the new priority is greater than or equal to the old priority
+        # or if the new priority is to cancel the coloring do we not return.
+        if { $oldpriority > $priority && $priority != 0 } { return }
+
+        set ::QListBox::priorities($windowname) [lreplace $prilist [expr $index - 1] [expr $index - 1] $priority]
+
 	killtags $windowname.t $index.0
-	if { [llength $args] > 1 } {
-	    $windowname.t tag configure $QListBox::tagindex -foreground [lindex $args 0] -background [lindex $args 1]
+	if { [llength $cols] > 1 } {
+	    $windowname.t tag configure $::QListBox::tagindex -foreground [lindex $cols 0] -background [lindex $cols 1]
 	} else {
-	    $windowname.t tag configure $QListBox::tagindex -foreground [lindex $args 0]
+	    $windowname.t tag configure $::QListBox::tagindex -foreground [lindex $cols 0]
 	}
-	$windowname.t tag add $QListBox::tagindex $index.0 $index.end
+	$windowname.t tag add $::QListBox::tagindex $index.0 $index.end
 	$windowname.t tag raise selected
-	incr QListBox::tagindex
+	incr ::QListBox::tagindex
     }
     proc select { windowname index } {
+        set ::QListBox::selected($windowname) $index
         incr index
         $windowname.t tag remove selected 0.0 end
         if { [$windowname.t get $index.0 $index.[string length $::dynamic::theme_windowlist_indent]]!=$::dynamic::theme_windowlist_indent } {
